/*
 * Project   : SIMUNO
 * Package   : simuno
 * File      : AminoacidTable.java
 * Created at: 30/09/2007 - 20:16:57
 */
package simuno;

import java.lang.Integer;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;

import simuno.imunology.objects.Aminoacid;

/**
 * This class holds an aminacid table. For each number from 1 to 20 we have mapped an specific
 * amino acid. They are sorted in alphabetical order. With this table we can build an atractiviness
 * table from the aminoacids.
 * simuno.AminoacidTable
 * @author Rodrigo Janurio da Silva
 * @version 1.0.0
 */
public class AminoacidTable
{
    private static HashMap<Integer, Aminoacid> table;
    
    {
        // Inserting the Alanine aminoacid
        AminoacidTable.table.put( new Integer( 1 )
                      , new Aminoacid( "Ala"
                                     , "Alanine"
                                     , "C3H7NO2"
                                     , 1.8
                                     , 0 ) );

        // Inserting the Arginine aminoacid
        AminoacidTable.table.put( new Integer( 2 )
                      , new Aminoacid( "Arg"
                                     , "Arginine"
                                     , "C6H14N4O2"
                                     , -4.5
                                     , 1 ) );
        
        // Inserting the Asparagine aminoacid
        AminoacidTable.table.put( new Integer( 3 )
                      , new Aminoacid( "Asn"
                                     , "Asparagine"
                                     , "C4H8N2O3"
                                     , -3.5
                                     , 0 ) );
        
        // Inserting the Aspartic acid aminoacid
        AminoacidTable.table.put( new Integer( 4 )
                      , new Aminoacid( "Asp"
                                     , "Aspartic acid"
                                     , "C4H7NO4"
                                     , -3.5
                                     , -1 ) );
        
        // Inserting the Cysteine aminoacid
        AminoacidTable.table.put( new Integer( 5 )
                      , new Aminoacid( "Cys"
                                     , "Cysteine"
                                     , "C3H7NO2S"
                                     , 2.5
                                     , 0 ) );
        
        // Inserting the Glutamic acid aminoacid
        AminoacidTable.table.put( new Integer( 6 )
                      , new Aminoacid( "Glu"
                                     , "Glutamic acid"
                                     , "C5H9NO4"
                                     , -3.5
                                     , -1 ) );
        
        // Inserting the Glutamine aminoacid
        AminoacidTable.table.put( new Integer( 7 )
                      , new Aminoacid( "Gln"
                                     , "Glutamine"
                                     , "C5H10N2O3"
                                     , -3.5
                                     , 0 ) );
        
        // Inserting the Glycine aminoacid
        AminoacidTable.table.put( new Integer( 8 )
                      , new Aminoacid( "Gly"
                                     , "Glycine"
                                     , "C2H5NO2"
                                     , -0.4
                                     , 0 ) );
        
        // Inserting the Histidine aminoacid
        AminoacidTable.table.put( new Integer( 9 )
                      , new Aminoacid( "His"
                                     , "Histidine"
                                     , "C6H9N3O2"
                                     , -3.2
                                     , 1 ) );
        
        // Inserting the Isoleucine aminoacid
        AminoacidTable.table.put( new Integer( 10 )
                      , new Aminoacid( "Ile"
                                     , "Isoleucine"
                                     , "C6H13N3O2"
                                     , 4.5
                                     , 0 ) );
        
        // Inserting the Leucine aminoacid
        AminoacidTable.table.put( new Integer( 11 )
                      , new Aminoacid( "Leu"
                                     , "Leucine"
                                     , "C6H13N3O2"
                                     , 3.8
                                     , 0 ) );
        
        // Inserting the Lysine aminoacid
        AminoacidTable.table.put( new Integer( 12 )
                                , new Aminoacid( "Lys"
                                               , "Lysine"
                                               , "C6H14N2O2"
                                               , -3.9
                                               , 1 ) );
        
        // Inserting the Methionine aminoacid
        AminoacidTable.table.put( new Integer( 13 )
                                , new Aminoacid( "Met"
                                               , "Methionine"
                                               , "C5H11NO2S"
                                               , 1.9
                                               , 0 ) );
        
        // Inserting the Phenylalanine aminoacid
        AminoacidTable.table.put( new Integer( 14 )
                                , new Aminoacid( "Phe"
                                               , "Phenylalanine"
                                               , "C9H11NO2"
                                               , 2.8
                                               , 0 ) );
        
        // Inserting the Proline aminoacid
        AminoacidTable.table.put( new Integer( 15 )
                                 , new Aminoacid( "Pro"
                                                , "Proline"
                                                , "C5H9NO2"
                                                , -1.6
                                                , 0 ) );
        
        // Inserting the Serine aminoacid
        AminoacidTable.table.put( new Integer( 16 )
                                , new Aminoacid( "Ser"
                                               , "Serine"
                                               , "C3H7NO3"
                                               , -0.8
                                               , 0 ) );
        
        // Inserting the Threonine aminoacid
        AminoacidTable.table.put( new Integer( 17 )
                                , new Aminoacid( "Thr"
                                               , "Threonine"
                                               , "C4H9NO3"
                                               , -0.7
                                               , 0 ) );
        
        // Inserting the Tryptophan aminoacid
        AminoacidTable.table.put( new Integer( 18 )
                      , new Aminoacid( "Trp"
                                     , "Tryptophan"
                                     , "C11H12N2O2"
                                     , -0.9
                                     , 0 ) );
        
        // Inserting the Tyrosine aminoacid
        AminoacidTable.table.put( new Integer( 19 )
                                , new Aminoacid( "Tyr"
                                               , "Tyrosine"
                                               , "C9H11NO3"
                                               , -1.3
                                               , 0 ) );
        
        // Inserting the Valine aminoacid
        AminoacidTable.table.put( new Integer( 19 )
                                , new Aminoacid( "Val"
                                               , "Valine"
                                               , "C5H11NO2"
                                               , 4.2
                                               , 0 ) );
    }
    
    /**
     * Returns to the caller the table built.
     * @return The table built.
     */
    public static HashMap<Integer, Aminoacid> getTable()
    {
        return AminoacidTable.table;
    }
    
    /**
     * Validates the three letter name passed as parameter
     * @param threeLetterName The name to be validated.
     * @return The aminoacid found or null if none is found.
     */
    public static Aminoacid searchByThreeLetterName( final String threeLetterName )
    {
        boolean                     found      = false;
        Aminoacid                   aminoacid  = null;
        HashMap<Integer, Aminoacid> aminoTable = AminoacidTable.getTable();
        Collection<Aminoacid>       aminoacids = aminoTable.values();
        Iterator<Aminoacid>         itAminos   = aminoacids.iterator();
        
        // Search for the aminoacid with the letters passed as parameter.
        while( ( itAminos.hasNext() ) &&
               ( !found ) )
        {
            aminoacid = itAminos.next();
            if ( threeLetterName.equals( aminoacid.getThreeLetterName() ) )
            {
                // Found it.
                found = true;
            }
        }
        
        return aminoacid;
    }
    
    public static Aminoacid searchByKey( final int key )
    {
        Aminoacid a = null;
        
        if ( AminoacidTable.table.containsKey( key ) )
        {
            a = AminoacidTable.table.get( new Integer( key ) );
        }
        
        return a;
    }
}
