/*
 * Project   : SIMUNO
 * Package   : simuno
 * File      : Main.java
 * Created at: 04/07/2007 - 20:13:10
 */
package simuno;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArrayList;

import simuno.imunology.Enviroment;
import simuno.imunology.objects.Aminoacid;
import simuno.imunology.objects.Protein;
import simuno.imunology.objects.cells.Cell;
import simuno.statistics.Statistics;
import simuno.view.SimulatorView;



/**
 * simuno.Main
 * @author Rodrigo
 * @version 1.0.0
 */
public class Main
{
    private static int numT4;
    
    private static int numAPC;
    
    private static int numAntigen;
    
    private static Protein protein;
    
    private static SimulatorView view;
    
    private static boolean running;
    
    static
    {
        Main.numAntigen = 0;
        Main.numAPC     = 0;
        Main.numT4      = 0;
        Main.protein    = null;
    }
    
	public static void main( String[] args )
	{
		view = new SimulatorView();
		view.display();
	}
	
	public static void run() {
        Enviroment env = Enviroment.getInstance();
        
	    // Cleanning the enviroment.
	    env.getCells().removeAll( env.getCells() );
	    env.getAntigens().removeAll( env.getAntigens() );
	    
        Main.numAntigen = view.getAntigens();
        Main.numAPC     = view.getAPC();
        Main.numT4      = view.getT4();
        Main.protein    = convertAminoacidList(view.getAminoacid());

	    try
	    {
	        // Creating anything in the enviroment.
	        env.createT4( Main.numT4 );
	        env.createAPCs( Main.numAPC );
	        env.createAntigen( Main.numAntigen, Main.protein );
	        
	        Statistics.getInstance().resetStatisticsPrinted();
	        //Statistics.getInstance().startCounter();
	    }
	    catch( NumberFormatException nfe )
	    {
	        nfe.printStackTrace();
	    }
	    catch( IllegalArgumentException iae )
	    {
	        iae.printStackTrace();
	    }
        running = true;
	}
	
    public static void reset() {
    	if (running) stop();
    	//reset fields
    	
    }
    
    public static void stop() {
    	running = false;
    	CopyOnWriteArrayList<Cell> cells = Enviroment.getInstance().getCells(); 
    	for ( Cell c : cells ) {
    		c.stop();
		}
    }
	
    public static boolean isRunning() {
    	return running;
    }
    
    public static SimulatorView getView() {
    	return view;
    }
    
	/**
	 * Parses the argument came from the command line.
	 * @param args The arguments to be parsed.
	 * @throws IllegalArgumentException If some of the arguments are invalid.
	 * @throws NumberFormatException if the numbered arguments are not number.
	 */
	/*private void parseArguments( String[] args )
        throws IllegalArgumentException
             , NumberFormatException
    {
        boolean passingAntigen   = false;
        boolean passingAminoacid = false;
        
        for( int i = 0; i < args.length; i++ )
        {
            // Parsing the number of T4 lymphocyte
            if ( args[i].equals( "-t4" ) )
            {
                if ( i + 1 >= args.length )
                { 
                    throw new IllegalArgumentException( "After a -t4 we should have a N > 0!" );
                }
                
                // Getting the number of lymphocytes T4.
                Main.numT4 = Integer.parseInt( args[++i] );
                
                if ( Main.numT4 <= 0 )
                {
                    throw new IllegalArgumentException( "The number of T4s can't be < 0!" ); 
                }
            }
            // Parsing the number of APCs
            else if ( args[i].equals( "-apc" ) )
            {
                if ( i + 1 >= args.length )
                { 
                    throw new IllegalArgumentException( "After a -apc we should have a N > 0!" );
                }
                
                // Getting the number of APCs.
                Main.numAPC = Integer.parseInt( args[++i] );
                
                if ( Main.numAPC <= 0 )
                {
                    throw new IllegalArgumentException( "The number of APCs can't be < 0!" ); 
                }
            }
            // Parsing the number of antigens
            else if ( args[i].equals( "-antigen" ) )
            {
                if ( i + 1 >= args.length )
                {
                    throw new IllegalArgumentException( "After a -antigen we should have a N > 0" );
                }
                
                // Getting the number of antigen 
                Main.numAntigen = Integer.parseInt( args[++i] );
                
                if ( Main.numAntigen <= 0 )
                {
                    throw new IllegalArgumentException( "The number of Antigens can't be < 0!" ); 
                }
                
                passingAntigen = true;
            }
            // Parsing the aminoacid sequence.
            else if ( args[i].equals( "-aminoacid" ) )
            {
                if ( i + 1 >= args.length )
                {
                    throw new IllegalArgumentException( "After a -aminoacid we should have an" +
                            " aminoacid sequence separeted by '|'!" ); 
                }
                
                // Getting the number of antigen
                String sequence = args[++i];
                
                if ( sequence.length() <= 0 )
                { 
                    throw new IllegalArgumentException( "The number of Antigens can't be < 0!" );
                }
                
                // parsing the aminoacid sequence into an ArrayList<Aminoacid>. 
                Main.protein = Main.convertAminoacidList( sequence );
                passingAminoacid  = true;
            }
            // Parsing to see if we should enter in graphic mode. 
            else if ( args[i].equals( "-graphic" ) )
            {
                // Start the graphic mode.
            }
        }
        
        // When we pass the number of antigens and do not pass their aminoacid 
        // sequence we raise an error
        if ( passingAntigen && !passingAminoacid )
        {
            throw new IllegalArgumentException( "When passin antigens we should also pass their " + 
                    "aminoacid sequence!" );
        }
    }*/
	
	/**
	 * Converts the aminoacid list passed in the command line in the format AA1|AA2|AA3|...|AAN to
	 * an ArrayList<Aminoacid> format. If we do aaSeq.get( k ).getType() we receive an "AAK" string.
	 * @param aminoacidList The list to be converted.
	 * @return The converted list.
	 */
	private static Protein convertAminoacidList( String aminoacidList )
	{
	    String[]             aminoacidTLNs = aminoacidList.split( "[|]" );
	    ArrayList<Aminoacid> aminoacids    = new ArrayList<Aminoacid>();
	    
	    for( String aminoacidTLN : aminoacidTLNs )
	    {
	        aminoacids.add( new Aminoacid( aminoacidTLN ) );
	    }
	    
	    return new Protein( aminoacids );
	}
}
