package simuno.imunology;

import java.util.ArrayList;
import java.util.concurrent.CopyOnWriteArrayList;

import simuno.imunology.objects.Antigen;
import simuno.imunology.objects.Protein;
import simuno.imunology.objects.cells.APC;
import simuno.imunology.objects.cells.Cell;
import simuno.imunology.objects.cells.T4;
import simuno.imunology.objects.substances.Substance;
import simuno.statistics.Statistics;

public final class Enviroment
{
    private ArrayList<Substance> substances;
    
    private CopyOnWriteArrayList<Cell> cells;
    
    private CopyOnWriteArrayList<Antigen> antigens;
    
    private static Enviroment instancia;
    
    
    private Enviroment()
    {
        this.cells      = new CopyOnWriteArrayList<Cell>();
        this.antigens   = new CopyOnWriteArrayList<Antigen>();
        this.substances = new ArrayList<Substance>();
    }
    
    public static Enviroment getInstance()
    {
        if (Enviroment.instancia == null)
        {
            Enviroment.instancia = new Enviroment();
        }
        
        return Enviroment.instancia;
    }
    
    public void add( final Substance sub )
    {
        this.substances.add( sub );
        // TODO Fazer mais alguma coisa.
    }

    /**
     * @return the substances
     */
    public final ArrayList<Substance> getSubstances()
    {
        return this.substances;
    }

    /**
     * @return the cells
     */
    public final CopyOnWriteArrayList<Cell> getCells()
    {
        return this.cells;
    }

    /**
     * @return the antigen.
     */
    public final CopyOnWriteArrayList<Antigen> getAntigens()
    {
        return this.antigens;
    }

    /**
     * @param hasAntigen the hasAntigen to set
     */
    public final void addAntigen( final Antigen antigen )
    {
        this.antigens.add( antigen );
        Thread t = new Thread( antigen );
        t.start();
    }
    
    public boolean hasAntigens()
    {
        return this.antigens.size() > 0;
    }
    
    public void removeAntigen( final Antigen antigen )
    {
        this.antigens.remove( antigen );
    }
    
    public void createAPCs( final int quantity )
    {
        for ( int i = 0; i < quantity; i++ )
        {
            APC apc = new APC( Statistics.getInstance() );
            this.getCells().add( apc );
            Thread t = new Thread( apc );
            t.start();
        }
    }
    
    public void createT4( final int quantity )
    {
        for ( int i = 0; i < quantity; i++ )
        {
            T4 t4 = new T4( Statistics.getInstance() );
            this.getCells().add( t4 );
            Thread t = new Thread( t4 );
            t.start();
        }
    }
    
    public void createAntigen( final int     quantity
                             , final Protein protein )
    {
        for ( int i = 0; i < quantity; i++ )
        {
//            Antigen a = new Antigen( protein );
            this.getAntigens().add( new Antigen( protein ) );
//            Thread t = new Thread( a );
//            t.start();
        }
    }

}
