/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : AminoacidSequence.java
 * Created at: 09/07/2007 - 12:56:50
 */
package simuno.imunology.objects;

import java.util.ArrayList;

/**
 * This class represents an aminoacid sequence. Both an Epitop and a Protein are a sequence of this
 * element.
 * simuno.imunology.objects.AminoacidSequence
 * @author Rodrigo Janurio da Silva.
 * @version 1.0.0
 */
public abstract class AminoacidSequence
{
    /** The sequence that forms a protein or an epitop. */
    protected ArrayList<Aminoacid> aminoacids;
    
    /**
     * Creates a new instance of AminoacidSequence
     */
    protected AminoacidSequence()
    {
        this.aminoacids = new ArrayList<Aminoacid>();
    }
    
    /**
     * Creates a new instance of AminoacidSequence. It generates the id and also sets the aminoacid
     * sequence as the one passed.
     * @param aminoacidSeq Aminoacid sequence to be set.
     * @throws IllegalArgumentException If the sequence passed is null. 
     */
    protected AminoacidSequence( final ArrayList<Aminoacid> aminoacids )
        throws IllegalArgumentException
    {
        this();
        this.validateAminoacidSequence( aminoacids );
        this.aminoacids = aminoacids;
    }

    /**
     * This method validates that the sequence passed as parametes is neither empty nor null.
     * @param aminoacidSequence Sequence to be validated.
     * @throws IllegalArgumentException If the sequence is null or empty.
     */
    private void validateAminoacidSequence( final ArrayList<Aminoacid> aminoacids )
        throws IllegalArgumentException
    {
        if ( null == aminoacids )
        {
            throw new IllegalArgumentException( "The aminoacid sequence can't be neither null" +
            		"nor empty." );
        }
    }

    /**
     * Returns the aminoacid sequence to the caller.
     * @return The aminoacid sequence to the caller.
     */
    public ArrayList<Aminoacid> getAminoacids()
    {
        return this.aminoacids;
    }

    /**
     * Sets the aminoacid sequence with the parameter passe.
     * @param aminoacidSequence The sequence to be set.
     */
    public void setAminoacids( ArrayList<Aminoacid> aminoacids )
    {
        this.validateAminoacidSequence( aminoacids );
        this.aminoacids = aminoacids;
    }
}
