/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : Antigen.java
 * Created at: 09/07/2007 - 13:16:12
 */
package simuno.imunology.objects;

import java.util.ArrayList;

/**
 * This class rpresents an entity not font in the human body. It starts the imunological response
 * when invades a human.
 * simuno.imunology.objects.Antigen
 * @author Rodrigo Janurio da Silva
 * @version 1.0.0
 */
public final class Antigen
    extends Thread
{
    /** The antigen id. */
    private long id;
    
    /** The antigen protein. */
    private Protein protein;
    
    /**
     * Creates a new instance of antigen.
     */
    public Antigen()
    {
        super();
    }

    /**
     * Creates a new instance of Antigen. It generates the id and also sets the aminoacid sequence
     * as the one passed.
     * @param aminoacidSeq Aminoacid sequence to be set.
     * @throws IllegalArgumentException If the sequence passed is null. 
     */
    public Antigen( Protein protein )
        throws IllegalArgumentException
    {
        this.protein = protein;
    }
    
    /**
     * Returns to the user the epitop id.
     * @return The user the protein id.
     */
    public final long getID()
    {
        return this.id;
    }
    
    /**
     * Returns the antigen aminoacids to the caller
     * @return The antigen aminoacids.
     */
    public final ArrayList<Aminoacid> getAminoacids()
    {
        return this.protein.getAminoacids();
    }
    
    /**
     * An antigen is equal to another only if the id is the same.
     * @return <b>true</b> if this equals to another. <b>false</b> otherwise.
     * @see java.lang.Object
     */
    public boolean equals( Object another )
    {
        boolean retCode = false;
        
        if ( ( null != another ) &&
        	 ( another instanceof Antigen ) )
        {
            if ( ( ( Antigen ) another).id == this.id )
            {
                retCode = true;
            }
        }
        
        return retCode;
    }
}

