/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : IdGenerator.java
 * Created at: 04/07/2007 - 20:03:34
 */
package simuno.imunology.objects;

import java.util.HashMap;

/**
 * A simple id generator for the objects or the SIMUNO project. It generates sequencial keys for
 * each object that needs an id.
 * simuno.imunology.objects.IdGenerator
 * @author Rodrigo Janurio da Silva
 * @version 1.0.0
 */
public class IdGenerator
{
	/**
	 * A object-id map. This means that for each object type there will be a different id.
	 */
	static HashMap<String, Long> objectToId;
	
	/* Inicializao statica. */
	static
	{
		IdGenerator.objectToId = new HashMap<String, Long>();
	}
	
	/**
	 * This method generates a new sequencial id from the old one.
	 * @return The generated id.
	 */
	public static long generateId( final Class<?> objeto )
	{
		long   value     = 0;
        String className = objeto.getName();
        
		if ( false == IdGenerator.objectToId.containsKey( className ) )
		{
			IdGenerator.objectToId.put( className, new Long( 0 ) );
		}
		
		value = IdGenerator.objectToId.get( className );
		value++;
		IdGenerator.objectToId.put( className, value );
		return value;
	}
}
