/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects.cells
 * File      : Cell.java
 * Created at: 10/08/2007 - 08:45:44
 */
package simuno.imunology.objects.cells;

import java.util.ArrayList;
import java.util.Observer;

import simuno.SimunoObservable;
import simuno.imunology.objects.Message;
import simuno.imunology.objects.conectors.Conector;

/**
 * This class represents an abstract cell. Lymphocytes and APCs are derived from this class.
 * simuno.imunology.objects.cells.Cell
 * @author Rodrigo Janurio da Silva
 * @author Marcos Tiago Garcia
 * @version 1.0.0
 */
public abstract class Cell
    extends SimunoObservable
    implements Runnable
{
    /** The cell conectors. */
    protected ArrayList<Conector> conectors;
    
    /** The observers for this cell. */
    protected ArrayList<Observer> observers;
    
    protected boolean stop;
    
    /**
     * Creates a new instance of Cell.
     * @param o The observer for the cell being created.
     * @throws NullPointerException if the parameter passed is null.
     */
    public Cell( Observer o )
    {
        this.conectors = new ArrayList<Conector>();
        this.observers = new ArrayList<Observer>();
        this.observers.add( o );
        this.addObserver( o );
        this.stop = false;
    }

    /*
     * (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public abstract void run();

    /**
     * This method represents a message receive. For each type of cell different messages are going
     * to be handled for them.
     * @param msg The message received.
     * @param cel The cell that sent the message.
     * @throws NullPointerException if some of the parameters are null.
     */
    public void receiveMessage( Message msg
                              , Cell    cel )
        throws NullPointerException
    {
        if ( ( null == msg ) ||
             ( null == cel ) )
        {
            throw new NullPointerException( "None of the parameters can be null!" );
        }
    }
    
    /**
     * This method simulates a cell reproduction.
     */
    public abstract void reproduce();
    
    public void stop() {
    	this.stop = true;
    }
}
