/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects.cells
 * File      : Lymphocyte.java
 * Created at: 10/08/2007 - 08:46:15
 */
package simuno.imunology.objects.cells;

import java.util.Observer;

import simuno.Sleeper;
import simuno.imunology.Enviroment;
import simuno.imunology.objects.Antigen;
import simuno.imunology.objects.Epitop;
import simuno.statistics.StatisticValue;
import simuno.statistics.StatisticsType;


/**
 * This class represents a lymphocyte in its natural state. This have not a specialization yet.
 * simuno.imunology.objects.cells.Lymphocyte
 * @author Rodrigo Janurio da Silva
 * @author Marcos Tiago Garcia.
 * @version 1.0.0
 */
public abstract class Lymphocyte
    extends Cell
{
    public Lymphocyte( Observer o )
    {
        super ( o );
    }
    
    public abstract boolean recognizeEpitop( final Epitop epitop )
        throws NullPointerException;
    
    public abstract boolean recognizeAntigen( final Antigen antigen )
        throws NullPointerException;
    
    /**
     * This method simulates an antigen killing. It simply removes the antigen from the enviroment.
     * @param antigen The antigen to be removed
     * @throws NullPointerException if the antigen passed as parameter is null.
     */
    public void killAntigen( final Antigen antigen )
        throws NullPointerException
    {
        if ( null != antigen )
        {
	        Enviroment env = Enviroment.getInstance();
	        env.getAntigens().remove( antigen );
	        Sleeper.sleepThread( 500 );
        	this.notifyObservers( new StatisticValue( StatisticsType.T4_NUM_ANTIGEN_KILLED
                                                    , new Integer( 1 ) ) );
        }
    }
}
