/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : TCR.java
 * Created at: 27/07/2007 - 23:47:06
 */
package simuno.imunology.objects.conectors;

import java.util.ArrayList;
import java.util.Date;
import java.util.Random;

import simuno.AminoacidTable;
import simuno.imunology.objects.Aminoacid;
import simuno.imunology.objects.IdGenerator;
import simuno.imunology.objects.Protein;

/**
 * Esta classe representa uma TCR. TCR  a parte de um linfcito ue se conecta  APC para
 * reconhecimento de um antgeno.  enviado um estmulo CD40 e aguardado um co-estmulo B7. Quando
 * este co-estmulo no  recebido o significa que o linfcito no reconheceu a entidade.
 * simuno.imunology.objects.TCR
 * @author Rodrigo Janurio da Silva
 * @version 1.0.0
 */
public final class TCR
    extends Conector
{
    /** Identificador da TCR */
    private long id;
    
    /** The TCR protein */
    private Protein protein;
    
    /**
     * Builds a new instance or TCR.
     */
    public TCR()
    {
        this.id = IdGenerator.generateId( this.getClass() );
        this.protein = new Protein( this.generateProteinRandomically() );
    }
    
    /**
     * Returns to the caller the TCR identificator
     * @return the id to the caller
     */
    public final long getId()
    {
        return this.id;
    }
    
    /**
     * 
     * @return
     */
    public final Protein getProtein()
    {
        return this.protein;
    }

    /**
     * 
     * @return
     */
    private ArrayList<Aminoacid> generateProteinRandomically()
    {
        Random               r    = new Random( ( new Date() ).getTime() );
        ArrayList<Aminoacid> list = new ArrayList<Aminoacid>();
        
        for( int i = 0; i < 100; i++ )
        {
            list.add( AminoacidTable.searchByKey( r.nextInt() ) );
        }
        
        return list;
    }
}
