/*
 * Project   : SIMUNO
 * Package   : simuno.statistics
 * File      : Statistics.java
 * Created at: 27/08/2007 - 19:14:03
 */
package simuno.statistics;

import java.io.PrintStream;
import java.util.Observable;
import java.util.Observer;

/**
 * simuno.statistics.Statistics
 * @author Rodrigo
 * @version 1.0.0
 */
public final class Statistics implements Observer
{
	private static Statistics instance; 
	
	private static boolean statisticsPrinted;
	
	public static final int NUM_VALUES = 8;

	
    private Integer apcNumAntigenEaten;
    private Integer apcNumEpitopPresented;
    private Integer apcNumB7Answers;
    private Integer t4NumCD28Sent;
    private Integer t4NumEpitopRecognized;
    private Integer t4NumAntigenRecognized;
    private Integer t4NumAntigenKilled;
    private Integer t4NumThatDidNotRecognizedAnything;
    
    static
    {
        Statistics.statisticsPrinted = false;
    }

    private Statistics()
    {
        initializeValues();
    }
    
    public static Statistics getInstance() {
    	if (instance == null) {
    		instance = new Statistics();
    	}
    	return instance;
    }
    
    public void startCounter() {
    	Thread t = new Thread(new StatisticsCounter());
    	t.start();
    }
    
    public void initializeValues()
    {
        this.apcNumB7Answers                   = 0;
        this.apcNumEpitopPresented             = 0;
        this.apcNumAntigenEaten                = 0;
        this.t4NumCD28Sent                     = 0;
        this.t4NumAntigenKilled                = 0;
        this.t4NumAntigenRecognized            = 0;
        this.t4NumEpitopRecognized             = 0;
        this.t4NumThatDidNotRecognizedAnything = 0;
    }
    
    public void resetStatisticsPrinted()
    {
        initializeValues();
        Statistics.statisticsPrinted = false;
    }
    
    /* (non-Javadoc)
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update( Observable o
                      , Object arg )
    {
        if( ! ( arg instanceof StatisticValue ) )
        {
            throw new IllegalArgumentException( "The type of arg should be StatisticValue!" );
        }
        
        StatisticValue value = ( StatisticValue ) arg;

    	synchronized( this )
    	{
	        switch( value.getType() )
	        {
	            case APC_NUM_ANTIGEN_EATEN:
	            {
	                this.apcNumAntigenEaten += value.getValue();
	            }
	            break;
	            
	            case APC_NUM_B7_ANSWERS:
	            {
	                this.apcNumB7Answers += value.getValue();
	            }
	            break;
	            
	            case APC_NUM_EPITOP_PRESENTED:
	            {
	                this.apcNumEpitopPresented += value.getValue();
	            }
	            break;
	            
	            case T4_NUM_CD28_SENT:
	            {
	                this.t4NumCD28Sent += value.getValue();
	            }
	            break;
	            
	            case T4_NUM_ANTIGEN_KILLED:
	            {
	                this.t4NumAntigenKilled += value.getValue();
	            }
	            break;
	            
	            case T4_NUM_ANTIGEN_RECOGNIZED:
	            {
	                this.t4NumAntigenRecognized += value.getValue();
	            }
	            break;
	            
	            case T4_NUM_EPITOP_RECOGNIZED:
	            {
	                this.t4NumEpitopRecognized += value.getValue();
	            }
	            break;
	            
	            case T4_NUM_THAT_DID_NOT_RECOGNIZED_ANYTHING:
	            {
	                this.t4NumThatDidNotRecognizedAnything += value.getValue();
	            }
	            break;
	        }
    	}
    }

    /**
     * @return the apcNumAntigenEaten
     */
    public final Integer getApcNumAntigenEaten()
    {
        return this.apcNumAntigenEaten;
    }

    /**
     * @return the apcNumEpitopPresented
     */
    public final Integer getApcNumEpitopPresented()
    {
        return this.apcNumEpitopPresented;
    }

    /**
     * @return the apcNumCD28Answers
     */
    public final Integer getApcNumB7Answers()
    {
        return this.apcNumB7Answers;
    }
    
    /**
     * @return the t4NumCD28Sent
     */
    public final Integer getT4NumCD28Sent()
    {
        return t4NumCD28Sent;
    }

    /**
     * @return the t4NumEpitopRecognized
     */
    public final Integer getT4NumEpitopRecognized()
    {
        return this.t4NumEpitopRecognized;
    }

    /**
     * @return the t4NumAntigenRecognized
     */
    public final Integer getT4NumAntigenRecognized()
    {
        return this.t4NumAntigenRecognized;
    }

    /**
     * @return the t4NumAntigenKilled
     */
    public final Integer getT4NumAntigenKilled()
    {
        return this.t4NumAntigenKilled;
    }

    /**
     * @return the t4NumThatDidNotRecognizedAnything
     */
    public final Integer getT4NumThatDidNotRecognizedAnything()
    {
        return this.t4NumThatDidNotRecognizedAnything;
    }
    
    public synchronized Integer[] getAllValues() {
    	Integer r[] = new Integer[NUM_VALUES];
    	r[0] = apcNumB7Answers;
    	r[1] = apcNumEpitopPresented;
    	r[2] = apcNumAntigenEaten;
    	r[3] = t4NumCD28Sent;
    	r[4] = t4NumAntigenKilled;
    	r[5] = t4NumAntigenRecognized;
    	r[6] = t4NumEpitopRecognized;
    	r[7] = t4NumThatDidNotRecognizedAnything;
    	return r;
    }
    
    public synchronized void printStatistics( PrintStream ps )
    {
    	if ( !Statistics.statisticsPrinted )
    	{
    		String s = "\n******************* ESTATSTICAS *******************\n"
    			     + "\tNmero de Antigenos Fagocitados   : " + this.apcNumAntigenEaten + "\n"
    			     + "\tNmero de Respostas B7 com Sucesso: " + this.apcNumB7Answers + "\n"
    			     + "\tNmero de Eptopos Apresentados   : " + this.apcNumEpitopPresented + "\n"
    			     + "\tNmero de Mensagens CD28 Enviadas : " + this.t4NumCD28Sent + "\n"
    			     + "\tNmero de Eptopos Reconhecidos   : " + this.t4NumEpitopRecognized + "\n"
    			     + "\tNmero de Antgenos Reconhecidos  : " + this.t4NumAntigenRecognized + "\n"
    			     + "\tNmero de Antgenos Destrudos    : " + this.t4NumAntigenKilled + "\n"
    			     + "******************* ESTATSTICAS *******************\n";
	    	ps.println( s );
	    	Statistics.statisticsPrinted = true;
    	}
    }
}
