
package simuno.view;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import simuno.Main;
import simuno.statistics.Statistics;

public class SimulatorView {

	private static Display display;
	private static Shell shell;
	private int width;
	private int height;
	private InputGroup inputGroup = null;
	private OutputGroup outputGroup = null;
	
	public SimulatorView() {
		display = new Display();
		shell = new Shell(display);
		width = 600;
		height = 300;
		initialize();
	}
	
	private void initialize() {
		shell.setText("SIMUNO: An Immunology Simulator v 1.0");

		shell.setBounds((display.getBounds().width - width)/2, 
				        (display.getBounds().height - height)/2, 
				        width, 
				        height);
		
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.makeColumnsEqualWidth = true;
		shell.setLayout(layout);

		inputGroup = new InputGroup(shell);
		outputGroup = new OutputGroup(shell);
		
		Button start = new Button(shell, SWT.FLAT);
		start.setText("Start / Stop");
		Button reset = new Button(shell, SWT.FLAT);
		reset.setText("Reset");

		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalAlignment = SWT.RIGHT;
		start.setLayoutData(gd);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		gd.horizontalAlignment = SWT.LEFT;
		reset.setLayoutData(gd);
		
		start.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (!Main.isRunning())
					Main.run();
				else
					Main.stop();
			}
			
		});
		reset.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				Main.reset();
			}
			
		});
		
		shell.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent arg0) {
				Main.stop();
			}
		});
		
	}
	
	public String getAminoacid() {
		return inputGroup.getAminoacid();
	}

	public Integer getAntigens() {
		return inputGroup.getAntigens();
	}

	public Integer getAPC() {
		return inputGroup.getAPC();
	}

	public Integer getT4() {
		return inputGroup.getT4();
	}
	
	public void display() {
		shell.open();
		while (!shell.isDisposed()) {
			if (!display.readAndDispatch()) {
				display.sleep();
			}
			if ( !shell.isDisposed() )
			{
			    Main.getView().setOutputValues( Statistics.getInstance().getAllValues() );
			}
		}
	}
	
	public void setOutputValues (Integer[] newValue) {
		outputGroup.setOutputValues(newValue);
	}
}

