/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : Gene.java
 * Created at: 04/07/2007 - 20:11:05
 */
package simuno.imunology.objects;

/**
 * This class represents human gene. 
 * simuno.imunology.objects.Gene
 * @author Rodrigo Janurio da Silva
 * @version 1.0.0
 */
public final class Gene
{
    /** The gene Id. */
    private long id;
    
    /**
     * The gene type.
     * TODO Verificar com prof. Eduardo se type faz sentido para genes.
     */
    private String type;
    
    /**
     * Says if the gene is active or not.
     * TODO Verificar com prof. Eduardo se esta noo faz sentido.
     */
    private boolean active;
    
    // TODO Sequencia de bases???.
	
	/**
	 * Creates a new instance of the Gene. An id is generated here.
	 */
	public Gene()
	{
		this.id    = IdGenerator.generateId( this.getClass() );
        this.type  = "";
        this.active = false;
	}
    
    /**
     * Creates a new instance of gene. We generate the gene id and also set the gene values in case
     * they are valid.
     * @param type The gene type to be set.
     * @param active Indicates if the gene is active or not..
     * @throws IllegalArgumentException If the type passed is not valid.
     */
    public Gene( final String  tipo
               , final boolean ativo )
        throws IllegalArgumentException
    {
        // Chamando o contrutor default para inicializar o id.
        this();
        this.validateType( tipo );
        this.type  = tipo;
        this.active = ativo;
    }

    /**
     * Method to validate the gene type. If it's empty or null it's invalid.
     * @param type The type to be validated.
     * @throws IllegalArgumentException If the type is null or empty.
     */
    private void validateType( final String type )
        throws IllegalArgumentException
    {
        // No  permitido que o type passado seja nulo ou vazio!!!
        if ( ( null == type ) ||
             ( type.trim().equals( "" ) ) )
        {
            throw new IllegalArgumentException( "The type can't be null or empty!" );
        }
    }
	
    /**
     * This method says if the gene is active or not.
     * @return <B>true</B> if the gene is active. <B>false</B> otherwise.
     */
    public final boolean isActive()
    {
        return this.active;
    }

    /**
     * Sets the gene active (true) or inactive (false).
     * @param active The value to be set..
     */
    public final void setAtivo( boolean active )
    {
        this.active = active;
    }

    /**
     * Method to return to the caller the gene type.
     * @return Returns the gene type.
     */
    public final String getType()
    {
        return this.type;
    }

    /**
     * Sets the gene type.
     * @param type The type value to be set.
     * @throws IllegalArgumentException If the type is not valid.
     */
    public final void setType( String type )
        throws IllegalArgumentException
    {
        this.validateType( type );
        this.type = type;
    }
    
    /**
     * Returns the gene id to the caller.
     * @return The gene id.
     */
    public final long getID()
    {
        return this.id;
    }
}
