/*
 * Project   : SIMUNO
 * Package   : simuno.imunology.objects
 * File      : Epitop.java
 * Created at: 09/07/2007 - 12:31:05
 */
package simuno.imunology.objects;

import java.util.ArrayList;


/**
 * Class representing a protein. A protein is an aminoacid sequence. All proteins capable to induce
 * the lymphocytic activit are called antigen.
 * simuno.imunology.objects.Protein
 * @author Rodrigo Janurio da Silva.
 * @version 1.0.0
 */
public class Protein
    extends AminoacidSequence
{
    /** Protein id. */
    private long id;
    
    /**
     * Creates a new instance of Protein.
     */
    public Protein()
    {
        super();
        this.id = IdGenerator.generateId( this.getClass() );
    }
    
    /**
     * Creates a new instance of protein. It generates the id and also sets the aminoacid sequence
     * as the one passed.
     * @param aminoacidSeq Aminoacid sequence to be set.
     * @throws IllegalArgumentException If the sequence passed is null. 
     */
    public Protein( final ArrayList<Aminoacid> aminoacidSeq )
        throws IllegalArgumentException
    {
        super( aminoacidSeq );
    }

    /**
     * Returns to the user the protein id.
     * @return The user the protein id.
     */
    public final long getId()
    {
        return id;
    }
}
